import 'package:flutter/material.dart';
import 'package:natinfo_flutter/providers/theme_provider.dart';
import 'package:provider/provider.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:natinfo_flutter/api/swagger.swagger.dart';

class NatinfDetail extends StatelessWidget {
  final Natinf natinf;
  const NatinfDetail({super.key, required this.natinf});

  @override
  Widget build(BuildContext context) {
    // Détermine couleur du background en fonction du type d'infraction ou type de mesure
    Color getBadgeColor(BuildContext context, String text) {
      final lower = text.trim().toLowerCase();
      final themeProvider = Provider.of<ThemeProvider>(context, listen: false);
      if (lower == 'obligatoire') return themeProvider.obligatoryColor;
      if (lower == 'facultatif') return themeProvider.facultativeColor;
      if (lower == 'non') return themeProvider.nonColor;
      if (lower.contains('crime')) return themeProvider.crimeColor;
      if (lower.contains('delit') || lower.contains('délit')) return themeProvider.delitColor;
      if (lower.contains('contravention')) return themeProvider.contraventionColor;
      return Colors.transparent;
    }

    bool hasBadge(String text) {
      final lower = text.trim().toLowerCase();
      return lower == 'obligatoire' ||
          lower == 'facultatif' ||
          lower == 'non' ||
          lower.contains('crime') ||
          lower.contains('delit') ||
          lower.contains('délit') ||
          lower.contains('contravention');
    }

    // À faire : le badge ne devrait pas être plus grand que l'espace du texte
    Widget buildBadge(String text) {
      if (!hasBadge(text)) return Text(text);
      return IntrinsicWidth(
        child: Container(
          padding: const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
          decoration: BoxDecoration(
            color: getBadgeColor(context, text),
            borderRadius: BorderRadius.circular(12),
          ),
          child: Text(
            text,
          ),
        ),
      );
    }

    final List<Widget> normalTiles = [];
    final List<Widget> bottomTiles = [];

    void addTile(String title, String? value, {bool useBadge = true}) {
      if (value == null || value.isEmpty || value == "N/A") return;
      final tile = ListTile(
        title: Text(
          title,
          style: const TextStyle(fontWeight: FontWeight.bold),
        ),
        subtitle: useBadge ? buildBadge(value) : Text(value),
      );
      if (value == "non") {
        bottomTiles.add(tile);
      } else {
        normalTiles.add(tile);
      }
    }

    String? modesProcesVerbalSubtitle;
    if (natinf.modesProcesVerbal != null) {
      if (natinf.modesProcesVerbal is List) {
        modesProcesVerbalSubtitle = (natinf.modesProcesVerbal as List<dynamic>)
            .map((mode) {
          switch (mode) {
            case 0:
              return 'CRI';
            case 1:
              return 'PVE';
            case 2:
              return 'PVO';
            default:
              return mode.toString();
          }
        }).join(' ou ');
      } else {
        modesProcesVerbalSubtitle = natinf.modesProcesVerbal.toString();
      }
    }

    addTile("Qualification de l'infraction", natinf.qualificationInfraction, useBadge: false);
    addTile("Nature de l'infraction", natinf.natureInfraction);
    addTile("Définie par", natinf.definiePar);
    addTile("Réprimée par", natinf.reprimeePar);
    addTile("Mode de rédaction", modesProcesVerbalSubtitle);
    addTile("Retrait points", natinf.retraitPoints?.toString());
    addTile("Contrôle alcoolémie", natinf.controleAlcoolemie?.value);
    addTile("Dépistage imprégnation alcoolique", natinf.depistageImpregnationAlcoolique?.value);
    addTile("Dépistage stupéfiants", natinf.depistageStupefiants?.value);
    addTile("Rétention permis de conduire", natinf.retentionPermisConduire?.value);
    addTile("Immobilisation du véhicule", natinf.immobilisationVehicule?.value);
    addTile("Mise en fourrière", natinf.miseEnFourriere?.value);
    addTile("Observations", natinf.observations, useBadge: false);

    final List<Widget> allTiles = [...normalTiles]..addAll(bottomTiles);

    final String webUrl = "https://natinfo.retiolus.net/natinf/${natinf.numeroNatinf}";
    allTiles.add(
      ListTile(
        title: const Text(
          "Accéder à plus de détails en ligne",
          style: TextStyle(
            color: Colors.blue,
            decoration: TextDecoration.underline,
          ),
        ),
        onTap: () async {
          final Uri url = Uri.parse(webUrl);
          if (await canLaunchUrl(url)) {
            await launchUrl(url, mode: LaunchMode.externalApplication);
          } else {
            ScaffoldMessenger.of(context).showSnackBar(
              const SnackBar(content: Text("Impossible d'ouvrir le lien")),
            );
          }
        },
      ),
    );

    return Scaffold(
      appBar: AppBar(
        title: Text("Détails du NATINF ${natinf.numeroNatinf ?? "N/A"}"),
      ),
      body: Padding(
        padding: const EdgeInsets.all(16.0),
        child: ListView(
          children: allTiles,
        ),
      ),
    );
  }
}
