import 'package:flutter/material.dart';
import 'package:natinfo_flutter/db/database_helper.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:natinfo_flutter/config/app_config.dart';
import 'package:provider/provider.dart';
import 'package:natinfo_flutter/providers/theme_provider.dart';
import 'package:flutter_colorpicker/flutter_colorpicker.dart';

class SettingsPage extends StatefulWidget {
  const SettingsPage({super.key});

  @override
  State<SettingsPage> createState() => _SettingsPageState();
}

class _SettingsPageState extends State<SettingsPage> {
  bool _forceOffline = AppConfig.forceOffline;

  @override
  void initState() {
    super.initState();
    _loadOfflineSetting();
  }

  Future<void> _loadOfflineSetting() async {
    final prefs = await SharedPreferences.getInstance();
    final offline = prefs.getBool('forceOffline') ?? false;
    setState(() {
      _forceOffline = offline;
      AppConfig.forceOffline = offline;
    });
  }

  Future<void> _saveOfflineSetting(bool value) async {
    final prefs = await SharedPreferences.getInstance();
    await prefs.setBool('forceOffline', value);
  }

  @override
  Widget build(BuildContext context) {
    final themeProvider = Provider.of<ThemeProvider>(context);
    return Scaffold(
      appBar: AppBar(title: const Text("Réglages")),
      body: SingleChildScrollView(
        child: Padding(
          padding: const EdgeInsets.all(16.0),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              const Text(
                "Mode hors ligne",
                style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold),
              ),
              SwitchListTile(
                title: const Text("Forcer le mode hors ligne"),
                subtitle: const Text(
                  "Activez cette option pour forcer la recherche en mode hors ligne, sans tenter de contacter le serveur.",
                ),
                value: _forceOffline,
                onChanged: (value) {
                  setState(() {
                    _forceOffline = value;
                    AppConfig.forceOffline = value;
                  });
                  _saveOfflineSetting(value);
                },
              ),
              const Divider(),
              const Text(
                "Personnalisation du thème",
                style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold),
              ),
              SwitchListTile(
                title: const Text("Mode sombre"),
                subtitle: const Text("Activez ou désactivez le mode sombre."),
                value: themeProvider.isDarkMode,
                onChanged: (value) {
                  themeProvider.toggleTheme(value);
                },
              ),
              const SizedBox(height: 16),
              const Text(
                "Couleur principale",
                style: TextStyle(fontSize: 16, fontWeight: FontWeight.w500),
              ),
              ElevatedButton(
                onPressed: () async {
                  Color pickerColor = themeProvider.primaryColor;
                  await showDialog(
                    context: context,
                    builder: (context) {
                      return AlertDialog(
                        title: const Text("Choisir une couleur"),
                        content: SingleChildScrollView(
                          child: ColorPicker(
                            pickerColor: pickerColor,
                            onColorChanged: (color) => pickerColor = color,
                            showLabel: true,
                            pickerAreaHeightPercent: 0.8,
                          ),
                        ),
                        actions: [
                          TextButton(
                            onPressed: () {
                              Navigator.of(context).pop();
                            },
                            child: const Text("Annuler"),
                          ),
                          TextButton(
                            onPressed: () {
                              themeProvider.updatePrimaryColor(pickerColor);
                              Navigator.of(context).pop();
                            },
                            child: const Text("Valider"),
                          ),
                        ],
                      );
                    },
                  );
                },
                child: const Text("Choisir la couleur"),
              ),
              TextButton(
                onPressed: () {
                  themeProvider.updatePrimaryColor(Colors.indigo);
                },
                child: const Text("Restaurer par défaut"),
              ),
              const Padding(
                padding: EdgeInsets.only(top: 8.0),
                child: Text(
                  "Sélectionnez la couleur principale qui sera utilisée dans l'interface.",
                ),
              ),
              const Divider(),
              const Text(
                "Couleurs des infractions",
                style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold),
              ),
              ListTile(
                leading: CircleAvatar(
                  backgroundColor: themeProvider.obligatoryColor,
                ),
                title: const Text("Obligatoire"),
                onTap: () async {
                  Color pickerColor = themeProvider.obligatoryColor;
                  await showDialog(
                    context: context,
                    builder: (context) {
                      return AlertDialog(
                        title: const Text(
                          "Choisir la couleur pour 'Obligatoire'",
                        ),
                        content: SingleChildScrollView(
                          child: ColorPicker(
                            pickerColor: pickerColor,
                            onColorChanged: (color) => pickerColor = color,
                            showLabel: true,
                            pickerAreaHeightPercent: 0.8,
                          ),
                        ),
                        actions: [
                          TextButton(
                            onPressed: () => Navigator.of(context).pop(),
                            child: const Text("Annuler"),
                          ),
                          TextButton(
                            onPressed: () {
                              themeProvider.updateObligatoryColor(pickerColor);
                              Navigator.of(context).pop();
                            },
                            child: const Text("Valider"),
                          ),
                        ],
                      );
                    },
                  );
                },
              ),
              ListTile(
                leading: CircleAvatar(
                  backgroundColor: themeProvider.facultativeColor,
                ),
                title: const Text("Facultatif"),
                onTap: () async {
                  Color pickerColor = themeProvider.facultativeColor;
                  await showDialog(
                    context: context,
                    builder: (context) {
                      return AlertDialog(
                        title: const Text(
                          "Choisir la couleur pour 'Facultatif'",
                        ),
                        content: SingleChildScrollView(
                          child: ColorPicker(
                            pickerColor: pickerColor,
                            onColorChanged: (color) => pickerColor = color,
                            showLabel: true,
                            pickerAreaHeightPercent: 0.8,
                          ),
                        ),
                        actions: [
                          TextButton(
                            onPressed: () => Navigator.of(context).pop(),
                            child: const Text("Annuler"),
                          ),
                          TextButton(
                            onPressed: () {
                              themeProvider.updateFacultativeColor(pickerColor);
                              Navigator.of(context).pop();
                            },
                            child: const Text("Valider"),
                          ),
                        ],
                      );
                    },
                  );
                },
              ),
              ListTile(
                leading: CircleAvatar(backgroundColor: themeProvider.nonColor),
                title: const Text("Non"),
                onTap: () async {
                  Color pickerColor = themeProvider.nonColor;
                  await showDialog(
                    context: context,
                    builder: (context) {
                      return AlertDialog(
                        title: const Text("Choisir la couleur pour 'Non'"),
                        content: SingleChildScrollView(
                          child: ColorPicker(
                            pickerColor: pickerColor,
                            onColorChanged: (color) => pickerColor = color,
                            showLabel: true,
                            pickerAreaHeightPercent: 0.8,
                          ),
                        ),
                        actions: [
                          TextButton(
                            onPressed: () => Navigator.of(context).pop(),
                            child: const Text("Annuler"),
                          ),
                          TextButton(
                            onPressed: () {
                              themeProvider.updateNonColor(pickerColor);
                              Navigator.of(context).pop();
                            },
                            child: const Text("Valider"),
                          ),
                        ],
                      );
                    },
                  );
                },
              ),
              ListTile(
                leading: CircleAvatar(
                  backgroundColor: themeProvider.crimeColor,
                ),
                title: const Text("Crime"),
                onTap: () async {
                  Color pickerColor = themeProvider.crimeColor;
                  await showDialog(
                    context: context,
                    builder: (context) {
                      return AlertDialog(
                        title: const Text("Choisir la couleur pour 'Crime'"),
                        content: SingleChildScrollView(
                          child: ColorPicker(
                            pickerColor: pickerColor,
                            onColorChanged: (color) => pickerColor = color,
                            showLabel: true,
                            pickerAreaHeightPercent: 0.8,
                          ),
                        ),
                        actions: [
                          TextButton(
                            onPressed: () => Navigator.of(context).pop(),
                            child: const Text("Annuler"),
                          ),
                          TextButton(
                            onPressed: () {
                              themeProvider.updateCrimeColor(pickerColor);
                              Navigator.of(context).pop();
                            },
                            child: const Text("Valider"),
                          ),
                        ],
                      );
                    },
                  );
                },
              ),
              ListTile(
                leading: CircleAvatar(
                  backgroundColor: themeProvider.delitColor,
                ),
                title: const Text("Délit"),
                onTap: () async {
                  Color pickerColor = themeProvider.delitColor;
                  await showDialog(
                    context: context,
                    builder: (context) {
                      return AlertDialog(
                        title: const Text("Choisir la couleur pour 'Délit'"),
                        content: SingleChildScrollView(
                          child: ColorPicker(
                            pickerColor: pickerColor,
                            onColorChanged: (color) => pickerColor = color,
                            showLabel: true,
                            pickerAreaHeightPercent: 0.8,
                          ),
                        ),
                        actions: [
                          TextButton(
                            onPressed: () => Navigator.of(context).pop(),
                            child: const Text("Annuler"),
                          ),
                          TextButton(
                            onPressed: () {
                              themeProvider.updateDelitColor(pickerColor);
                              Navigator.of(context).pop();
                            },
                            child: const Text("Valider"),
                          ),
                        ],
                      );
                    },
                  );
                },
              ),
              ListTile(
                leading: CircleAvatar(
                  backgroundColor: themeProvider.contraventionColor,
                ),
                title: const Text("Contravention"),
                onTap: () async {
                  Color pickerColor = themeProvider.contraventionColor;
                  await showDialog(
                    context: context,
                    builder: (context) {
                      return AlertDialog(
                        title: const Text(
                          "Choisir la couleur pour 'Contravention'",
                        ),
                        content: SingleChildScrollView(
                          child: ColorPicker(
                            pickerColor: pickerColor,
                            onColorChanged: (color) => pickerColor = color,
                            showLabel: true,
                            pickerAreaHeightPercent: 0.8,
                          ),
                        ),
                        actions: [
                          TextButton(
                            onPressed: () => Navigator.of(context).pop(),
                            child: const Text("Annuler"),
                          ),
                          TextButton(
                            onPressed: () {
                              themeProvider.updateContraventionColor(
                                pickerColor,
                              );
                              Navigator.of(context).pop();
                            },
                            child: const Text("Valider"),
                          ),
                        ],
                      );
                    },
                  );
                },
              ),
              TextButton(
                onPressed: () {
                  themeProvider.resetInfractionColors();
                },
                child: const Text("Restaurer les couleurs par défaut"),
              ),
              const Padding(
                padding: EdgeInsets.only(top: 8.0),
                child: Text(
                  "Personnalisez les couleurs utilisées pour afficher les informations sur les infractions.",
                ),
              ),
              const Divider(),
              const Text(
                "Données téléchargées",
                style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold),
              ),
              ListTile(
                leading: const Icon(Icons.delete_forever),
                title: const Text("Supprimer les données locales"),
                subtitle: const Text("Effacez toutes les données synchronisées depuis le serveur."),
                onTap: () async {
                  final confirmed = await showDialog<bool>(
                    context: context,
                    builder: (context) {
                      return AlertDialog(
                        title: const Text("Confirmer la suppression"),
                        content: const Text("Êtes-vous sûr de vouloir supprimer toutes les données locales ? Cette opération est irréversible."),
                        actions: [
                          TextButton(
                            onPressed: () => Navigator.of(context).pop(false),
                            child: const Text("Annuler"),
                          ),
                          TextButton(
                            onPressed: () => Navigator.of(context).pop(true),
                            child: const Text("Supprimer"),
                          ),
                        ],
                      );
                    },
                  );
                  if (confirmed ?? false) {
                    try {
                      await DatabaseHelper().clearOfflineData();
                      ScaffoldMessenger.of(context).showSnackBar(
                        const SnackBar(content: Text("Données locales supprimées avec succès.")),
                      );
                    } catch (e) {
                      ScaffoldMessenger.of(context).showSnackBar(
                        SnackBar(content: Text("Erreur lors de la suppression : $e")),
                      );
                    }
                  }
                },
              ),
            ],
          ),
        ),
      ),
    );
  }
}