import 'dart:async';
import 'package:natinfo_flutter/api/swagger.swagger.dart';
import 'package:natinfo_flutter/db/database_helper.dart';

class NatinfRepository {
  late final Swagger _apiClient;
  late final DatabaseHelper _dbHelper;

  Future<void> init() async {
    _dbHelper = DatabaseHelper();
    await _dbHelper.initDb();
    _apiClient = Swagger.create(
      baseUrl: Uri.parse('http://natinfo.retiolus.net/api'),
    );
  }

  Future<bool> hasLocalData() async {
    return !await _dbHelper.isEmpty();
  }

  Future<void> fetchInitialData({
    void Function(String)? onStatus,
    void Function(double)? onProgress,
    void Function(int)? onPhase,
  }) async {
    // Téléchargement depuis l'API
    onStatus?.call("Téléchargement des données depuis l'API...");
    onPhase?.call(1);
    final response = await _apiClient
        .natinfoGet()
        .timeout(const Duration(seconds: 60));
    if (response.isSuccessful && response.body != null) {
      final List<Natinf> list = response.body as List<Natinf>;
      
      // Importation dans la base locale
      onStatus?.call("Téléchargement terminé. Importation des données dans la base locale...");
      onPhase?.call(2);
      final total = list.length;
      final stopwatch = Stopwatch()..start();
      for (int i = 0; i < total; i++) {
        final natinf = list[i];
        await _dbHelper.insertOrUpdateNatinf(natinf);
        // Calcul du progrès de l'importation
        final progressFraction = (i + 1) / total;
        final progressPercent = (progressFraction * 100).toStringAsFixed(0);
        final elapsed = stopwatch.elapsed.inSeconds;
        final speed = elapsed > 0 ? ((i + 1) / elapsed).toStringAsFixed(1) : "calcul...";
        onStatus?.call("Importation: $progressPercent% (${i + 1}/$total) - $speed items/s");
        onProgress?.call(progressFraction);
      }
      onStatus?.call("Importation terminée.");
    } else {
      throw Exception("Erreur lors du téléchargement initial: ${response.statusCode}");
    }
  }

  Future<List<Natinf>> searchNatinf(String query, {void Function(String)? onStatus, bool forceOffline = false}) async {
    if (forceOffline) {
      onStatus?.call("Mode hors ligne forcé : recherche immédiate en BD locale...");
      final localResults = await _dbHelper.getNatinfByQuery(query);
      if (localResults.isNotEmpty) {
        onStatus?.call("Résultats trouvés en BD locale (mode hors ligne forcé).");
        return localResults;
      }
      throw Exception("Aucun NATINF trouvé en BD locale (mode hors ligne forcé).");
    }
    
    try {
      onStatus?.call("Envoi de la requête API NATINFo...");
      final response = await _apiClient
          .natinfoSearchGet(q: query)
          .timeout(const Duration(seconds: 10));
      onStatus?.call("Réponse API reçue: ${response.statusCode}");
      if (response.isSuccessful && response.body != null) {
        List<Natinf> list = response.body as List<Natinf>;
        if (list.isNotEmpty) {
          onStatus?.call("Réponse API valide, mise à jour BD locale...");
          for (final natinf in list) {
            await _dbHelper.insertOrUpdateNatinf(natinf);
          }
          return list;
        } else {
          throw Exception("Aucun NATINF trouvé sur le serveur.");
        }
      } else {
        throw Exception("Erreur serveur : ${response.statusCode}");
      }
    } on TimeoutException catch (_) {
      onStatus?.call("Timeout (10s), passage en mode hors ligne, recherche locale...");
      final localResults = await _dbHelper.getNatinfByQuery(query);
      if (localResults.isNotEmpty) {
        onStatus?.call("Résultats trouvés en BD locale (mode hors ligne activé en raison d'un timeout API).");
        return localResults;
      }
      rethrow;
    } catch (e) {
      onStatus?.call("Erreur API, passage en mode hors ligne, recherche locale...");
      final localResults = await _dbHelper.getNatinfByQuery(query);
      if (localResults.isNotEmpty) {
        onStatus?.call("Résultats trouvés en BD (mode hors ligne activé à cause d'une erreur API: ${e.toString()}).");
        return localResults;
      }
      rethrow;
    }
  }
}
